Package: TDbasedUFEadv
Type: Package
Title: Advanced package of tensor decomposition based unsupervised
        feature extraction
Version: 1.12.1
Language: en-US
Authors@R: 
    person("Y-h.", "Taguchi", ,
    "tag@granular.com", role = c("aut", "cre"),
    comment = c(ORCID  = "0000-0003-0867-8986"))
Description: 
    This is an advanced version of TDbasedUFE, which is a  comprehensive package
    to perform Tensor decomposition based unsupervised feature extraction.
    In contrast to TDbasedUFE which can perform simple the feature selection and 
    the multiomics analyses, this package can perform more complicated and 
    advanced features, but they are not so popularly required. Only users who 
    require more specific features can make use of its functionality. 
biocViews: GeneExpression, FeatureExtraction, MethylationArray,
        SingleCell, Software
License: GPL-3
Encoding: UTF-8
Imports: TDbasedUFE, Biobase, GenomicRanges, utils, rTensor, methods,
        graphics, stats, hash, shiny
BugReports: https://github.com/tagtag/TDbasedUFEadv/issues
URL: https://github.com/tagtag/TDbasedUFEadv
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), RTCGA.rnaseq,
        RTCGA.clinical, BiocStyle, MOFAdata, STRINGdb, enrichR,
        enrichplot, DOSE, gson, ggplot2
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
git_url: https://git.bioconductor.org/packages/TDbasedUFEadv
git_branch: RELEASE_3_23
git_last_commit: 3703f3a
git_last_commit_date: 2026-05-04
Repository: Bioconductor 3.23
Date/Publication: 2026-05-05
NeedsCompilation: no
Packaged: 2026-05-06 00:45:57 UTC; biocbuild
Author: Y-h. Taguchi [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0867-8986>)
Maintainer: Y-h. Taguchi <tag@granular.com>

# Generated by roxygen2: do not edit by hand

export(computeSVD)
export(prepareCondDrugandDisease)
export(prepareCondTCGA)
export(prepareTensorRect)
export(prepareTensorfromList)
export(prepareTensorfromMatrix)
export(prepareexpDrugandDisease)
export(selectFeatureProj)
export(selectFeatureRect)
export(selectFeatureTransRect)
export(transSVD)
import(Biobase)
import(TDbasedUFE)
importFrom(GenomicRanges,GRanges)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,par)
importFrom(hash,keys)
importFrom(methods,is)
importFrom(methods,new)
importFrom(rTensor,as.tensor)
importFrom(rTensor,hosvd)
importFrom(shiny,actionButton)
importFrom(shiny,fluidPage)
importFrom(shiny,h3)
importFrom(shiny,mainPanel)
importFrom(shiny,observeEvent)
importFrom(shiny,plotOutput)
importFrom(shiny,renderPlot)
importFrom(shiny,runApp)
importFrom(shiny,shinyApp)
importFrom(shiny,sidebarLayout)
importFrom(shiny,sidebarPanel)
importFrom(shiny,stopApp)
importFrom(stats,filter)
importFrom(stats,optim)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,var)
importFrom(utils,read.csv)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# TDbasedUFEadv

<!-- badges: start -->
<!-- badges: end -->

The goal of TDbasedUFEadv is to …

## Installation



``` r
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("TDbasedUFEadv")
```

You can install the latest release of TDbasedUFEadv from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("tagtag/TDbasedUFEadv@0.99.0-1")
```

You can install the development version of TDbasedUFEadv from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("tagtag/TDbasedUFEadv",build_vignettes = TRUE)
```

or if it does not work, try

``` r
# install.packages("devtools")
devtools::install_github("tagtag/TDbasedUFEadv")
```

## Introduction

It is an advanced version of
[TDbasedUFE](https://bioconductor.org/packages/devel/bioc/html/TDbasedUFE.html).
Thus people who would like to use TDbasedUFEadv, please first install
and check TDbasedUFE.

## Vignettes

For theoretical memo

vignette(“Explanation_of_TDbasedUFEadv”)

How to use it.

vignette(“How_to_use_TDbasedUFEadv”)

vignette(“Enrichment”)

For more theoretical background

vignette(“TDbasedUFEadv”)

drug_response.txt is included in
http://lifeome.net/supp/drug_response/
First, download 
http://lifeome.net/supp/drug_response/response.zip
and unzip. Then you can get  drug_response.txt.

drug_response.txt is an ascii text file.
It includes fifteen columns, "cancers", "patient.arr", "drug.name", "drug.id",
"response", "start.time", "end.time", "pathological_time", "procurement_method",
"procurement_time", "nte_time", "nte_pharm", "nte_surgery"", separated by tab.
Among them we do need association between "patient.arr" and "drug.name"", 
although  "cancers" are also requied to idenfied files that gene expression 
is stored.

#' @title Prepare Sample label for TCGA data
#'
#' @param Multi_sample list of sample ids
#' @param Clinical List of clinical data matrix from RTCGA.clinical
#' @param ID_column_of_Multi_sample Column numbers used for conditions
#' @param ID_column_of_Clinical Column numbers that include corresponding
#'  sample ids in clinical data
#'
#' @return list of sample labels
#' @export
#'
#' @examples
#' library(RTCGA.clinical)
#' library(RTCGA.rnaseq)
#' Clinical <- list(BLCA.clinical, BRCA.clinical, CESC.clinical, COAD.clinical)
#' Multi_sample <- list(
#'   BLCA.rnaseq[seq_len(100), 1, drop = FALSE],
#'   BRCA.rnaseq[seq_len(100), 1, drop = FALSE],
#'   CESC.rnaseq[seq_len(100), 1, drop = FALSE],
#'   COAD.rnaseq[seq_len(100), 1, drop = FALSE]
#' )
#' ID_column_of_Multi_sample <- c(770, 1482, 773, 791)
#' ID_column_of_Clinical <- c(20, 20, 12, 14)
#' cond <- prepareCondTCGA(
#'   Multi_sample, Clinical,
#'   ID_column_of_Multi_sample, ID_column_of_Clinical
#' )
prepareCondTCGA <- function(Multi_sample, Clinical,
                            ID_column_of_Multi_sample,
                            ID_column_of_Clinical) {
  # Argument check
  stopifnot("`Multi_sample` must be an list." = is.list(Multi_sample))
  stopifnot("`Clinical` must be an list." = is.list(Multi_sample))
  stopifnot(
    "`ID_column_of_Clinical` must be a vector" = is.vector(ID_column_of_Clinical)
  )
  stopifnot(
    "`ID_column_of_Multi_sample` must be a vector" = is.vector(ID_column_of_Multi_sample)
  )
  stopifnot(
    "Length must be common among `Multi_sample` " = var(unlist(lapply(list(
      Multi_sample, Clinical,
      ID_column_of_Multi_sample,
      ID_column_of_Clinical
    ), length))) == 0
  )
  #
  Cond <- rep(list(NA), length(Multi_sample))
  for (i in seq_along(Multi_sample))
  {
    index <- match(
      tolower(substring(Multi_sample[[i]][, 1], 1, 12)),
      Clinical[[i]][, ID_column_of_Clinical[i]]
    )
    Cond[[i]] <- Clinical[[i]][index, ID_column_of_Multi_sample[i]]
  }
  return(Cond)
}

require(RTCGA.rnaseq)
require(RTCGA.clinical)
require(Biobase)
require(MOFAdata)

test_that("prepareexpDrugandDisease works", {  
  Cancer_cell_lines <- list(ACC.rnaseq, BLCA.rnaseq, BRCA.rnaseq)
  Drug_and_Disease <- prepareexpDrugandDisease(Cancer_cell_lines)
  expect_true(is.list(Drug_and_Disease))
  expDrug <<- Drug_and_Disease$expDrug
  expDisease <- Drug_and_Disease$expDisease
})
test_that("prepareCondDrugandDisease work", {
  Cond <- prepareCondDrugandDisease(expDrug)
  expect_true(is.table(Cond))
})
test_that("prepareTensorfromMatrix works", {
  data("CLL_data")
  data("CLL_covariates")
  Z <<- prepareTensorfromMatrix(
    t(CLL_data$Drugs[seq_len(200), seq_len(50)]),
    t(CLL_data$Methylation[seq_len(200), seq_len(50)])
  )
  expect_true(is.array(Z))
})
test_that("prepareTensorRect works", {
  Z <<- prepareTensorRect(
    sample = colnames(CLL_data$Drugs)[seq_len(50)],
    feature = list(
      Drugs = rownames(CLL_data$Drugs)[seq_len(200)],
      Methylatiion = rownames(CLL_data$Methylation)[seq_len(200)]
    ),
    sampleData = list(CLL_covariates[, 1][seq_len(50)]),
    value = Z
  )
  expect_true(is(Z, "TensorRect"))
})
test_that("selectFeatureTransRect works", {
  HOSVD <- computeHosvd(Z)
  cond <- list(attr(Z, "sampleData")[[1]], NULL, NULL)
  index_all <- selectFeatureTransRect(HOSVD, cond,
    de = c(0.01, 0.01),
    input_all = 8
  ) # batch mode
  expect_true(is.list(index_all))
})

test_that("computeSVD works", {
  SVD <<- computeSVD(t(CLL_data$Drugs), t(CLL_data$Methylation))
  expect_true(is.list(SVD))
})
test_that("transSVD works", {
  Z <- CLL_data$Drugs %*% t(CLL_data$Methylation)
  expect_true(is.matrix(Z))
  sample <- colnames(CLL_data$Methylation)
  Z <<- prepareTensorRect(
    sample = sample,
    feature = list(rownames(CLL_data$Drugs), rownames(CLL_data$Methylation)),
    value = array(NA, dim(Z)), sampleData = list(CLL_covariates[, 1])
  )
  SVD <<- transSVD(SVD)
  expect_true(is.list(SVD))
})
test_that("selectFeatureRect works", {
    cond <- list(NULL, attr(Z, "sampleData")[[1]], attr(Z, "sampleData")[[1]])
  index_all <- selectFeatureRect(SVD, cond, de = c(0.5, 0.5), input_all = 6) # batch mode
  expect_true(is.list(index_all))
})
test_that(" prepareTensorfromList",{
  Z <- prepareTensorfromList(CLL_data, 10L)
  expect_true(is.array(Z))
})
test_that("prepareTensorfromList works", {
  Multi <<- list(
    BLCA.rnaseq[seq_len(100), 1 + seq_len(1000)],
    BRCA.rnaseq[seq_len(100), 1 + seq_len(1000)],
    CESC.rnaseq[seq_len(100), 1 + seq_len(1000)],
    COAD.rnaseq[seq_len(100), 1 + seq_len(1000)]
  )
  Z <<- prepareTensorfromList(Multi, 10L)
  expect_true(is.array(Z))
  })
test_that("prepareCondTCGA works", {
  Z <<- aperm(Z, c(2, 1, 3))
  Clinical <- list(BLCA.clinical, BRCA.clinical, CESC.clinical, COAD.clinical)
  Multi_sample <- list(
    BLCA.rnaseq[seq_len(100), 1, drop = FALSE],
    BRCA.rnaseq[seq_len(100), 1, drop = FALSE],
    CESC.rnaseq[seq_len(100), 1, drop = FALSE],
    COAD.rnaseq[seq_len(100), 1, drop = FALSE]
  )
  # patient.stage_event.tnm_categories.pathologic_categories.pathologic_m
  ID_column_of_Multi_sample <- c(770, 1482, 773, 791)
  # patient.bcr_patient_barcode
  ID_column_of_Clinical <- c(20, 20, 12, 14)
  Z <<- PrepareSummarizedExperimentTensor(
    feature = colnames(ACC.rnaseq)[1 + seq_len(1000)],
    sample = array("", 1), value = Z,
    sampleData = prepareCondTCGA(
      Multi_sample,
      Clinical, ID_column_of_Multi_sample,
      ID_column_of_Clinical
    )
  )
  expect_true(is(Z, "SummarizedExperimentTensor"))
})
  test_that("selectFeatureProj  works", {
      HOSVD <- computeHosvd(Z)
  cond <- attr(Z, "sampleData")
  index <- selectFeatureProj(HOSVD, Multi, cond, de = 1e-3, input_all = 3) # Batch mode
  expect_true(is.list(index))
})

#' @title Generating gene expression of drug treated cell lines and a disease
#' cell line
#'
#' @param Cancer_cell_lines <- list(ACC.rnaseq,BLCA.rnaseq,BRCA.rnaseq)
#'  list that includes individual data set from RTCGA.rnaseq 
#'
#' @return list of expDrug and expDisease 
#' @export
#'
#' @examples
#'  \donttest{
#' library(RTCGA.rnaseq)
#' Cancer_cell_lines <- list(ACC.rnaseq,BLCA.rnaseq,BRCA.rnaseq)
#' Drug_and_Disease <- prepareexpDrugandDisease(Cancer_cell_lines)
#' }
prepareexpDrugandDisease <- function(Cancer_cell_lines) {
  # Argument check
  stopifnot(
    "`Cancer_cell_lines` must be an list." = is.list(Cancer_cell_lines)
  )
  #
  dir <- system.file("extdata", package = "TDbasedUFEadv")
  TCGA <- read.csv(file.path(dir, "drug_response.txt"), sep = "\t")
  TCGAdb <- unique(TCGA$cancers)
  if (length(TCGAdb) > 16) TCGAdb <- TCGAdb[-16]
  RTCGA_all <- matrix(NA,
    nrow = dim(TCGA)[1],
    ncol = dim(Cancer_cell_lines[[1]])[2]
  )
  Cancer_cell_lines_p <- rep(list(NA), length(Cancer_cell_lines))
  toTCGA <- function(x) {
    ID <- lapply(strsplit(as.character(x[, 1]), "-"), "[", seq_len(3))
    ID <- t(data.frame(ID))
    ID <- paste(ID[, 1], ID[, 2], ID[, 3], sep = "-")
    return(x[ID %in% TCGA$patient.arr, ])
  }
  Cancer_cell_lines_p <- lapply(Cancer_cell_lines, toTCGA)
  
  expDrug <- lapply(seq_along(Cancer_cell_lines_p), function(i) {
      data.frame(Cancer_cell_lines_p[[i]])}
  )
  expDrug <- do.call("rbind", expDrug)
  expDrug <- .convertTCGA(expDrug)
  ID <- lapply(
    strsplit(as.character(Cancer_cell_lines[[3]][, 1]), "-"),
    "[", seq_len(3)
  )
  ID <- t(data.frame(ID))
  ID <- paste(ID[, 1], ID[, 2], ID[, 3], sep = "-")
  expDisease <- Cancer_cell_lines[[3]][!(ID %in% TCGA$patient.arr), ]
  expDisease <- .convertTCGA(expDisease)
  return(list(expDrug = expDrug, expDisease = expDisease))
}

#' @title Generate tensor from two matrices
#'
#' @param matrix1 the first input matrix
#' @param matrix2 the second input matrix
#'
#' @return A tensor generated from the first and second matrices 
#' @export
#'
#' @examples
#' Z <- prepareTensorfromMatrix(matrix(runif(100),10),matrix(runif(100),10))
prepareTensorfromMatrix <- function(matrix1, matrix2) {
  # Argument Check
  stopifnot("`matrix1` must be a matrix." = is.matrix(matrix1))
  stopifnot("`matrix2` must be a matrix." = is.matrix(matrix2))
  #
  col_num1 <- dim(matrix1)[2]
  col_num2 <- dim(matrix2)[2]
  matrix1[is.na(matrix1)] <- 0
  matrix2[is.na(matrix2)] <- 0
  Z <- apply(
    cbind(matrix1, matrix2), 1,
    function(x) {
      outer(
        x[seq_len(col_num1)],
        x[col_num1 + seq_len(col_num2)]
      )
    }
  )
  dim(Z) <- c(col_num1, col_num2, dim(Z)[2])
  Z <- aperm(Z, c(3, 1, 2))
  return(Z)
}

#' @title Prepare tensor from a list that includes multiple profiles
#'
#' @param Multi  a list that includes multiple profiles
#' @param proj_dim the number of projection dimensions
#'
#' @return a tensor as a bundle of singular value vectors obtained by 
#' applying  SVD to individual omics
#' @export
#'
#' @examples
#' library(MOFAdata)
#' data("CLL_data")
#' data("CLL_covariates")
#' Z <- prepareTensorfromList(CLL_data,10L)
prepareTensorfromList <- function(Multi, proj_dim) {
  # Argument check
  stopifnot("`Multi` must be a list." = is.list(Multi))
  stopifnot("`Proj_dim` must be an integer." = is.integer(proj_dim))
  #
  Multi_list <- lapply(Multi, function(x) {
    x[is.na(x)] <- 0
    return(x)
  })
  SVD_lst <- lapply(
    Multi_list,
    function(x) {
      SVD <- svd(x, proj_dim)
      return(t(SVD$v[, seq_len(proj_dim)]))
    }
  )
  Z <- array(unlist(SVD_lst), c(dim(SVD_lst[[1]]), length(Multi)))
  return(Z)
}

#' @title Convert SVD to that for the case where 
#' samples are shared between two matrices
#'
#' @param SVD input SVD object generated from computeSVD function
#'
#' @return converted SVD objects
#' @export
#'
#' @examples
#' matrix1 <- matrix(runif(200),20)
#' matrix2 <- matrix(runif(400),20)
#' SVD <- computeSVD(matrix1,matrix2)
#' SVD <- transSVD(SVD)
transSVD <- function(SVD) {
  # Augument check
  stopifnot("`SVD` must be a list." = is.list(SVD))
  #
  u0 <- SVD$SVD$u
  v0 <- SVD$SVD$v
  SVD$SVD$u <- scale(SVD$u)
  SVD$SVD$v <- scale(SVD$v)
  SVD$u <- scale(u0)
  SVD$v <- scale(v0)
  return(SVD)
}

#' @title Prepare condition matrix for expDrug
#'
#' @param expDrug input gene expression profile
#'
#' @return Condition matrix for expDrug
#' @export
#'
#' @examples
#' \donttest{
#' library(RTCGA.rnaseq)
#' Cancer_cell_lines <- list(ACC.rnaseq,BLCA.rnaseq,BRCA.rnaseq)
#' Drug_and_Disease <- prepareexpDrugandDisease(Cancer_cell_lines)
#' Cond <- prepareCondDrugandDisease(Drug_and_Disease$expDrug)
#' }
prepareCondDrugandDisease <- function(expDrug) {
  # Arugument Check
  stopifnot(
    "`expDrug` must be an ExpressionSet." = is(expDrug, "ExpressionSet")
  )
  #
  dir <- system.file("extdata", package = "TDbasedUFEadv")
  TCGA <- read.csv(file.path(dir, "drug_response.txt"), sep = "\t")
  Cond <- table(TCGA$patient.arr, TCGA$drug.name)
  ID <- lapply(strsplit(as.character(colnames(expDrug)), "-"), "[", seq_len(3))
  ID <- t(data.frame(ID))
  ID <- paste(ID[, 1], ID[, 2], ID[, 3], sep = "-")
  Cond <- Cond[match(ID, rownames(Cond)), ]
  return(Cond)
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom stats var
#' @importFrom methods new is
#' @importFrom utils read.csv
#' @import TDbasedUFE
#' @import Biobase
#' @importFrom shiny fluidPage sidebarLayout  sidebarPanel actionButton mainPanel 
#' @importFrom shiny plotOutput observeEvent runApp shinyApp stopApp renderPlot h3
#' @importFrom GenomicRanges GRanges
#' @importFrom rTensor hosvd as.tensor
#' @importFrom stats pchisq p.adjust optim
#' @importFrom graphics hist par arrows boxplot abline
#' @importFrom hash keys
#' @importFrom stats filter
## usethis namespace: end
NULL

#' @title Select feature when projection strategy is employed for the
#'  case where features are shared with multiple omics profiles
#'
#' @param HOSVD HOSVD 
#' @param Multi list of omics profiles, row: sample, column: feature
#' @param cond list of conditions for individual omics profiles
#' @param de initial value for optimization of standard deviation
#' @param p0 Threshold P-value
#' @param breaks The number of bins of histogram of P-values
#' @param input_all The number of selected feature. if null, interactive mode
#' is activated 
#'
#' @return list composed of logical vector that represent which features are selected and p-values
#' @export
#'
#' @examples
#' library(TDbasedUFE)
#' Multi <- list(matrix(runif(1000),10),matrix(runif(1000),10),
#' matrix(runif(1000),10),matrix(runif(1000),10))
#' Z <- prepareTensorfromList(Multi,10L)
#' Z <- aperm(Z,c(2,1,3))
#' Z <- PrepareSummarizedExperimentTensor(feature =as.character(1:10),
#'                                       sample=array("",1),value=Z)
#' HOSVD <- computeHosvd(Z)
#' cond <- rep(list(rep(1:2,each=5)),4)
#' index <- selectFeatureProj(HOSVD,Multi,cond,de=0.1,input_all=2)
selectFeatureProj <-
  function(HOSVD, Multi, cond, de = 1e-4, p0 = 0.01, breaks = 100L,
           input_all = NULL) {
    # Augument check
    stopifnot("`HOSVD` must be a list." = is.list(HOSVD))
    stopifnot("`Multi` must be a list." = is.list(Multi))
    stopifnot("`de` must be a numeric." = is.numeric(de))
    stopifnot("`p0` must be a numeric." = is.numeric(p0))
    stopifnot("`breaks` must be a integer." = is.integer(breaks))
    stopifnot("`input_all` must be a vector." = is.vector(input_all) |
      is.null(input_all))
    #
    interact <- FALSE
    Multi_list <- lapply(
      Multi,
      function(x) {
        data.matrix(x) %*% data.matrix(HOSVD$U[[1]])
      }
    )
    if (is.null(input_all)) {
      interact <- TRUE
      j <- 1
      ui <- fluidPage(
        sidebarLayout(
          sidebarPanel(
            h3("Select one with prefarable dependence"),
            actionButton(inputId = "action", label = "Next"),
            actionButton(inputId = "prev", label = "Prev"),
            actionButton(inputId = "select", label = "Select")
          ),
          mainPanel(
            plotOutput("plot")
          )
        )
      )
      server <- function(input, output) {
        observeEvent(input$action, {
          if (j < dim(HOSVD$U[[1]])[2]) j <<- j + 1
        })
        observeEvent(input$prev, {
          if (j != 1) {
            j <<- j - 1
          }
        })
        observeEvent(input$select, {
          input_all <<- j
          stopApp()
        })
        output$plot <- renderPlot({
          input$action
          input$prev
          par(mfrow = c(length(cond), 1))
          par(mai = c(0.3, 0.2, 0.2, 0.2))
          for (i in seq_along(cond))
          {
            boxplot(Multi_list[[i]][, j] ~ cond[[i]],
              main = paste(j, i, sep = "-")
            )
            abline(0, 0, col = 2, lty = 2)
          }
          par(mfrow = c(1, 1))
        })
      }
      app <- shinyApp(ui, server)
      runApp(app)
      input_all <- j
    } else {
      par(mfrow = c(length(cond), 1))
      par(mai = c(0.3, 0.2, 0.2, 0.2))
      for (i in seq_along(cond))
      {
        boxplot(Multi_list[[i]][, input_all] ~ cond[[i]],
          main = paste(input_all, i, sep = "-")
        )
        abline(0, 0, col = 2, lty = 2)
      }
      par(mfrow = c(1, 1))
    }
    th <- function(sd, breaks, p0) {
      P2 <- pchisq((u / sd)^2, 1, lower.tail = FALSE)
      hc <- hist(1 - P2, breaks = breaks, plot = FALSE)
      return(sd(hc$count[seq_len(sum(hc$breaks
      < 1 - min(P2[p.adjust(P2, "BH") > p0])))]))
    }
    u <- HOSVD$U[[1]][, input_all]
    sd <- optim(de, function(x) {
      th(x, breaks, p0)
    },
    control = list(warn.1d.NelderMead = FALSE)
    )$par
    sd1 <- seq(0.1 * sd, 2 * sd, by = 0.1 * sd)
    th0 <- apply(matrix(sd1, ncol = 1), 1, function(x) {
      th(x, breaks, p0)
    })
    P2 <- pchisq((u / sd)^2, 1, lower.tail = FALSE)
    ui <- fluidPage(
      sidebarLayout(
        sidebarPanel(
          actionButton(inputId = "action", label = "Next")
        ),
        mainPanel(
          plotOutput("plot")
        )
      )
    )
    server <- function(input, output) {
      observeEvent(input$action, {
        stopApp()
      })
      output$plot <- renderPlot({
        input$action
        par(mfrow = c(1, 2))
        plot(sd1, th0, type = "o")
        arrows(sd, max(th0), sd, min(th0), col = 2)
        hist(1 - P2, breaks = breaks)
        par(mfrow = c(1, 1))
      })
    }
    app <- shinyApp(ui, server)
    if (interact) {
      runApp(app)
    } else {
      par(mfrow = c(1, 2))
      plot(sd1, th0, type = "o")
      arrows(sd, max(th0), sd, min(th0), col = 2)
      hist(1 - P2, breaks = breaks)
      par(mfrow = c(1, 1))
    }
    index <- p.adjust(P2, "BH") < p0
    index_all <- list(index = index, p.value = P2)
    return(index_all)
  }

#' @title Select features for a tensor generated from two matrices that
#'  share samples.
#'
#' @param HOSVD  HOSVD 
#' @param cond list of conditions
#' @param de initial values for optimization of standard deviation
#' @param p0 threshold value for the significance
#' @param breaks number of bins of the histogram of P-values
#' @param input_all The selected singular value vectors attributed to samples.
#' if NULL, interactive mode
#'
#' @return list of logical vector that represent if the individual features
#' are selected and P-values. 
#' @export 
#'
#' @examples
#' library(TDbasedUFE)
#' set.seed(0)
#' matrix1 <- matrix(runif(1000),20) #row features, column samples
#' matrix2 <- matrix(runif(2000),40) #row features, column samples
#' Z <- prepareTensorfromMatrix(t(matrix1),t(matrix2))
#' Z <- prepareTensorRect(sample=as.character(seq_len(50)),
#' feature=list(as.character(seq_len(20)),as.character(seq_len(40))),
#' sampleData=list(rep(seq_len(2),each=25)),value=Z)
#' HOSVD <- computeHosvd(Z)
#' cond <- list(attr(Z,"sampleData")[[1]],NULL,NULL)
#'index_all <- selectFeatureTransRect(HOSVD,cond,de=c(0.1,0.1),
#'input_all=2,p0=1e-10)
selectFeatureTransRect <- function(HOSVD, cond, de = rep(1e-4, 2), p0 = 0.01,
                                   breaks = 100L, input_all = NULL) {{
  # Augument check
  stopifnot("`HOSVD` must be a list." = is.list(HOSVD))
  stopifnot("`cond` must be a list." = is.list(cond))
  stopifnot("`de` must be a numeric." = is.numeric(de))
  stopifnot("`p0` must be a numeric." = is.numeric(p0))
  stopifnot("`breaks` must be a integer." = is.integer(breaks))
  stopifnot("`input_all` must be a vector." = is.vector(input_all) |
    is.null(input_all))
  #
  interact <- FALSE
  if (is.null(input_all)) {
    interact <- TRUE
    j <- 1
    ui <- fluidPage(
      sidebarLayout(
        sidebarPanel(
          h3("Select one with prefarable dependence"),
          actionButton(inputId = "action", label = "Next"),
          actionButton(inputId = "prev", label = "Prev"),
          actionButton(inputId = "select", label = "Select")
        ),
        mainPanel(
          plotOutput("plot")
        )
      )
    )
    server <- function(input, output) {
      observeEvent(input$action, {
        if (j < dim(HOSVD$U[[1]])[2]) j <<- j + 1
      })
      observeEvent(input$prev, {
        if (j != 1) {
          j <<- j - 1
        }
      })
      observeEvent(input$select, {
        input_all <<- j
        stopApp()
      })
      output$plot <- renderPlot({
        input$action
        input$prev
        for (i in seq_along(cond))
        {
          boxplot(HOSVD$U[[1]][, j] ~ cond[[1]], main = j)
          abline(0, 0, col = 2, lty = 2)
        }
      })
    }
    app <- shinyApp(ui, server)
    runApp(app)
    input_all <- j
  } else {
    for (i in seq_along(cond))
    {
      boxplot(HOSVD$U[[1]][, input_all] ~ cond[[1]], main = input_all)
      abline(0, 0, col = 2, lty = 2)
    }
  }
  th <- function(sd, breaks, p0) {
    P2 <- pchisq((u / sd)^2, 1, lower.tail = FALSE)
    hc <- hist(1 - P2, breaks = breaks, plot = FALSE)
    return(sd(hc$count[seq_len(sum(hc$breaks
    < 1 - min(P2[p.adjust(P2, "BH") > p0])))]))
  }
  index_all <- rep(list(NA))
  for (i in seq_len(2))
  {
    u <- HOSVD$U[[i + 1]][, input_all]
    sd <- optim(de[i], function(x) {
      th(x, breaks, p0)
    },
    control = list(warn.1d.NelderMead = FALSE)
    )$par
    sd1 <- seq(0.1 * sd, 2 * sd, by = 0.1 * sd)
    th0 <- apply(matrix(sd1, ncol = 1), 1, function(x) {
      th(x, breaks, p0)
    })
    P2 <- pchisq((u / sd)^2, 1, lower.tail = FALSE)
    ui <- fluidPage(
      sidebarLayout(
        sidebarPanel(
          actionButton(inputId = "action", label = "Next")
        ),
        mainPanel(
          plotOutput("plot")
        )
      )
    )
    server <- function(input, output) {
      observeEvent(input$action, {
        stopApp()
      })
      output$plot <- renderPlot({
        input$action
        par(mfrow = c(1, 2))
        plot(sd1, th0, type = "o")
        arrows(sd, max(th0), sd, min(th0), col = 2)
        hist(1 - P2, breaks = breaks)
        par(mfrow = c(1, 1))
      })
    }
    app <- shinyApp(ui, server)
    if (interact) {
      runApp(app)
    } else {
      par(mfrow = c(1, 2))
      plot(sd1, th0, type = "o")
      arrows(sd, max(th0), sd, min(th0), col = 2)
      hist(1 - P2, breaks = breaks)
      par(mfrow = c(1, 1))
    }

    index <- p.adjust(P2, "BH") < p0
    index_all[[i]] <- list(index = index, p.value = P2)
  }
  return(index_all)
}}

#' @title Prepare tensor generated from two matrices that share samples
#'
#' @param sample Character vector of sample names
#' @param feature list of features from two matrices
#' @param value array, contents of 
#' @param featureRange Genomic Ranges to be associated with features
#' @param sampleData List of conditional labeling associated with samples
#'
#' @return Tensor generated from two matrices that share samples
#' @export
#'
#' @examples
#' matrix1 <- matrix(runif(1000),200) #row features, column samples
#' matrix2 <- matrix(runif(2000),400) #row features, column samples
#' Z <- prepareTensorfromMatrix(t(matrix1),t(matrix2))
#' Z <- prepareTensorRect(sample=as.character(seq_len(50)),
#' feature=list(as.character(seq_len(200)),as.character(seq_len(400))),
#' sampleData=list(rep(seq_len(2),each=25)),value=Z)
prepareTensorRect <- function(
    sample, feature, value,
    featureRange = GRanges(NULL), sampleData = list(NULL)) {
  # Argument check
  stopifnot("`sample` must be a character." = is.character(sample))
  stopifnot("`feature` must be a list." = is.list(feature))
  stopifnot("`value` must be a array." = is.array(value))
  stopifnot("`featureRange` must be an GRanges." = is(featureRange, "GRanges"))
  stopifnot("`sampleData` must be a list." = is.list(sampleData))
  #
  new("TensorRect",
    sample = sample,
    feature = feature, value = value,
    featureRange = featureRange, sampleData = sampleData
  )
}

#' @title Class definitions
#'
#' @slot sample character. 
#' @slot feature list. 
#' @slot value array. 
#' @slot featureRange GRanges. 
#' @slot sampleData list. 
setClass("TensorRect",slot=list(sample="character",
                                                feature="list",value="array",
                                                featureRange="GRanges",sampleData="list"))

#' @title Select features through the selection  of singular value vectors
#'
#' @param SVD SVD computed from matrix generated by partial summation of a tensor
#' @param cond Condition to select singular value vectors
#' @param de  Initial values to be used for optimization of standard deviation  
#' @param p0  Threshold value for the significance
#' @param breaks Number of bins of histogram of P-values
#' @param input_all The ID of selected singular value vectors. If it is null, 
#' interactive mode is activated.
#'
#' @return List of lists that includes P-vales as well as if individual
#'  features selected.
#' @export
#'
#' @examples
#' set.seed(0)
#' matrix1 <- matrix(runif(2000),200)
#' matrix2 <- matrix(runif(4000),200)
#' SVD <- computeSVD(matrix1,matrix2)
#' index_all <- selectFeatureRect(SVD,
#' list(NULL,rep(seq_len(2),each=5),rep(seq_len(2),each=10)),de=rep(0.5,2),
#' input_all=1)
selectFeatureRect <- function(SVD, cond, de = rep(1e-4, 2), p0 = 0.01,
                              breaks = 100L, input_all = NULL) {
  # Augument check
  stopifnot("`SVD` must be a list." = is.list(SVD))
  stopifnot("`cond` must be a list." = is.list(cond))
  stopifnot("`de` must be a numeric." = is.numeric(de))
  stopifnot("`p0` must be a numeric." = is.numeric(p0))
  stopifnot("`breaks` must be a integer." = is.integer(breaks))
  stopifnot("`input_all` must be a vector." = is.vector(input_all) |
    is.null(input_all))
  #
  interact <- FALSE
  if (is.null(input_all)) {
    interact <- TRUE
    j <- 1
    ui <- fluidPage(
      sidebarLayout(
        sidebarPanel(
          h3("Select one with prefarable dependence"),
          actionButton(inputId = "action", label = "Next"),
          actionButton(inputId = "prev", label = "Prev"),
          actionButton(inputId = "select", label = "Select")
        ),
        mainPanel(
          plotOutput("plot")
        )
      )
    )

    server <- function(input, output) {
      observeEvent(input$action, {
        if (j < dim(SVD$SVD$u)[2]) j <<- j + 1
      })
      observeEvent(input$prev, {
        if (j != 1) {
          j <<- j - 1
        }
      })
      observeEvent(input$select, {
        input_all <<- j
        stopApp()
      })
      output$plot <- renderPlot({
        input$action
        input$prev
        par(mfrow = c(1, 2))
        boxplot(SVD$SVD$u[, j] ~ cond[[2]], main = j)
        abline(0, 0, col = 2, lty = 2)
        boxplot(SVD$SVD$v[, j] ~ cond[[3]], main = j)
        abline(0, 0, col = 2, lty = 2)
        par(mfrow = c(1, 1))
      })
    }
    app <- shinyApp(ui, server)
    runApp(app)
  } else {
    par(mfrow = c(1, 2))
    boxplot(SVD$SVD$u[, input_all] ~ cond[[2]], main = input_all)
    abline(0, 0, col = 2, lty = 2)
    boxplot(SVD$SVD$v[, input_all] ~ cond[[3]], main = input_all)
    abline(0, 0, col = 2, lty = 2)
    par(mfrow = c(1, 1))
  }
  th <- function(sd, breaks, p0) {
    P2 <- pchisq((u / sd)^2, 1, lower.tail = FALSE)
    hc <- hist(1 - P2, breaks = breaks, plot = FALSE)
    return(sd(hc$count[seq_len(sum(hc$breaks
    < 1 - min(P2[p.adjust(P2, "BH") > p0])))]))
  }
  index_all <- rep(list(NA))
  for (i in seq_len(2))
  {
    u <- scale(SVD[[i + 1]][, input_all])
    sd <- optim(de[i], function(x) {
      th(x, breaks, p0)
    },
    control = list(warn.1d.NelderMead = FALSE)
    )$par
    sd1 <- seq(0.1 * sd, 2 * sd, by = 0.1 * sd)
    th0 <- apply(matrix(sd1, ncol = 1), 1, function(x) {
      th(x, breaks, p0)
    })
    P2 <- pchisq((u / sd)^2, 1, lower.tail = FALSE)
    if (interact) {
      ui <- fluidPage(
        sidebarLayout(
          sidebarPanel(
            actionButton(inputId = "action", label = "Next")
          ),
          mainPanel(
            plotOutput("plot")
          )
        )
      )
      server <- function(input, output) {
        observeEvent(input$action, {
          stopApp()
        })
        output$plot <- renderPlot({
          input$action
          par(mfrow = c(1, 2))
          plot(sd1, th0, type = "o")
          arrows(sd, max(th0), sd, min(th0), col = 2)
          hist(1 - P2, breaks = breaks)
          par(mfrow = c(1, 1))
        })
      }
      app <- shinyApp(ui, server)
      runApp(app)
    } else {
      par(mfrow = c(1, 2))
      plot(sd1, th0, type = "o")
      arrows(sd, max(th0), sd, min(th0), col = 2)
      hist(1 - P2, breaks = breaks)
      par(mfrow = c(1, 1))
    }
    index <- p.adjust(P2, "BH") < p0
    index_all[[i]] <- list(index = index, p.value = P2)
  }
  return(index_all)
}
    

#' Title Perform SVD toward reduced matrix generated from a tensor with 
#' partial summation
#'
#' @param matrix1 The first original matrix that generates a tensor
#' @param matrix2 The second original matrix that generates a tensor
#' @param dim  The number of singular value vectors to be computed
#' @param scale If matrix should be scaled or not
#'
#' @return Singular value vectors attributed to two sets of objects associated 
#' with singular value vectors attributed to features, by multiplying  
#' @export
#'
#' @examples
#' matrix1 <- matrix(runif(200),20)
#' matrix2 <- matrix(runif(400),20)
#' SVD <- computeSVD(matrix1,matrix2)
computeSVD <- function(matrix1, matrix2, dim = 10L, scale = TRUE) {
  # Argument check
  stopifnot("`matrix1` must be a matrix." = is.matrix(matrix1))
  stopifnot("`matrix2` must be a matrix." = is.matrix(matrix2))
  stopifnot("`dim` must be a integer." = is.integer(dim))
  stopifnot("`scale` must be a logical." = is.logical(scale))
  stopifnot(
    "# of rows must be common between `matrix1` and `matrix2`" 
        = nrow(matrix1) == nrow(matrix2)
  )
  #
  matrix1[is.na(matrix1)] <- 0
  matrix2[is.na(matrix2)] <- 0
  Z <- t(matrix1) %*% matrix2
  if (scale) Z <- Z / mean(Z)
  SVD <- svd(Z, dim, dim)
  u <- matrix1 %*% SVD$u
  v <- matrix2 %*% SVD$v
  return(list(SVD = SVD, u = u, v = v))
}

.convertTCGA <- function(dataSet) {
    if (!is.data.frame(dataSet)) {
        stop("'dataSet' must be a data.frame.", call. = FALSE)
    }

    if (ncol(dataSet) < 2L) {
        stop("'dataSet' must have at least two columns.", call. = FALSE)
    }

    rnaseqMatrix <- t(as.matrix(dataSet[, -1, drop = FALSE]))
    colnames(rnaseqMatrix) <- as.character(dataSet[[1]])

    Biobase::ExpressionSet(assayData = rnaseqMatrix)
}

---
title: "Explanation of TDbasedUFEadv"
author:
- name: Y-h. Taguchi
  affiliation:  Department of Physics, Chuo University, Tokyo 112-8551, Japan
  email: tag@granular.com
output:   
    BiocStyle::html_document:
    toc: true
bibliography: references.bib
vignette: >
  %\VignetteIndexEntry{Explanation of TDbasedUFEadv}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
#library(TDbasedUFEadv)
```

# Introduction 

Since TDbasedUFEadv is an advanced package from TDbasedUFE [@TDbasedUFE], please master the 
contents in TDbasedUFE prior to the trial of this package. 

## Motivations

Since the publication of the book [@Taguchibook] describing the methodology, 
I have published numerous papers using this method. In spite of that, very 
limited number of researcher used this method, possibly because of unfamiliarity
with the mathematical concepts used in this methodology, tensors. 
Thus I decided to develop the packages by which users can use the methods 
without detailed knowledge about the tensor. 

## What differs from related packages.

The main purpose of this package is to select features (typically genes) based 
upon provided omics data sets. In this sense, apparently the functionality of this
package is similar to [DESeq2](https://doi.org/doi:10.18129/B9.bioc.DESeq2) or [limma](https://doi.org/doi:10.18129/B9.bioc.limma), which have  functionality that can 
identify differentially expressed genes. In contrast to those supervised methods,
the present method is unsupervised one, which provides users what kind of 
profiles are observed over samples, and users are advised to select one of
favorite features by which features are selected. In addition to this, the
present method is suitable to small number of samples associated with large 
number of features. Since this situation is very common in genomics, the 
present method is supposed to be suitable to be applied to genomics, although
it does not look liked the methods very specific to genomics science. 
Actually, we have published the number of papers using the methods implemented 
in the present package. I hope that one can make use of this package for his/her
own researches. 


# Integrated analysis of two omics data sets


## When features are shared.

### Full tensor 

![Tensor decomposition towards tensor generated from two matrices](./fig23.jpg)

Suppose we have two omics profiles
$$
x_{ij} \in \mathbb{R}^{N \times M} \\
x_{ik} \in \mathbb{R}^{N \times K}
$$
that represent values of $i$th feature  of $j$th and $k$th objects, respectively 
(i.r., these two profiles share the features).
In this case, we generate a tensor, $x_{ijk}$, by the product of two profiles as
$$
x_{ijk} =  x_{ij}x_{ik} \in \mathbb{R}^{N \times M \times K}
$$

and HOSVD was applied to $x_{ijk}$ to get
$$
x_{ijk} = \sum_{\ell_1} \sum_{\ell_2} \sum_{ell_3} G(\ell_1 \ell_2 \ell_3) 
u_{\ell_1 i} u_{\ell_2 j} u_{\ell_3 k}
$$
After that we can follow the standard procedure to select features $i$s
associated with the desired properties represented by the selected singular
value vectors, $u_{\ell_2 j}$ and $u_{\ell_3 k}$, attributed to objects,
$j$s and $k$s.

### Matrix generated by partial summation 

![After partial summation of tensor](./fig24.jpg)

In the above, we dealt with full tensor. It is often difficult to treat full 
tensor, since it is as large as $N \times M times K$. In this case, we can take
the alternative approach. In order that we define reduced matrix with taking 
partial summation
$$
x_{jk} = \sum_i x_{ijk}
$$
and apply SVD to $x_{jk}$ as
$$
x_{jk} = \sum_\ell \lambda_\ell u_{\ell j} v_{\ell k}
$$
and singular value vectors attributed to samples as
$$
u^{(j)}_{\ell i} = \sum_j u_{\ell j} x_{ij} \\
u^{(k)}_{\ell i} = \sum_k v_{\ell k} x_{ik}
$$
In this case, singular value vectors are attributed separately to features
associated with objects $j$ and $k$, respectively.

The feature selection can be done using these singular value vectors associated
with selected singular value vectors attributed to samples, $j$ and $k$.

## When samples are shared.

### Full tensor

![Tensor decomposition towards tensor generated from two matrices](./fig25.jpg)

In the case where not features but samples are shared between two omics data, 
we can do something similar.
$$
x_{ij} \in \mathbb{R}^{N \times M} \\
x_{kj} \in \mathbb{R}^{K \times M}
$$


In this case, we generate a tensor, $x_{ijk}$, by the product of two profiles as
$$
x_{ijk} =  x_{ij}x_{kj} \in \mathbb{R}^{N \times M \times K}
$$

and HOSVD was applied to $x_{ijk}$ to get
$$
x_{ijk} = \sum_{\ell_1} \sum_{\ell_2} \sum_{ell_3} G(\ell_1 \ell_2 \ell_3) 
u_{\ell_1 i} u_{\ell_2 j} u_{\ell_3 k}
$$
After that we can follow the standard procedure to select features $i$s and $k$s
associated with the desired properties represented by the selected singular
value vectors, $u_{\ell_2 j}$, attributed to objects, $j$s.

###  Matrix generated from partial summation

![After partial summation of tensor](./fig26.jpg)

In the above, we dealt with full tensor. It is often difficult to treat full 
tensor, since it is as large as $N \times M times K$. In this case, we can take
the alternative approach. In order that we define reduced matrix with taking 
partial summation
$$
x_{ik} = \sum_j x_{ijk}
$$
and apply SVD to $x_{jk}$ as
$$
x_{ik} = \sum_\ell \lambda_\ell u_{\ell i} v_{\ell k}
$$
$i$s and $k$s are selected with $u_{\ell i}$ and $v_{\ell k}$, respectively. 
Singular value vectors attributed to samples can be computed as
$$
u^{(i)}_{\ell j} = \sum_i u_{\ell i} x_{ij} \\
u^{(k)}_{\ell j} = \sum_k v_{\ell k} x_{kj}
$$

# Integrated analysis using projection

Here we would like to propose an alternative strategy to integrate multiple 
tensors using projection with SVD.

## When samples are shared.

![Projection when samples are shared](./fig27.jpg)

Suppose we have multiomics data as
$$
x_{ijk} \in \mathbb{R}^{N_k \times M \times K}
$$
for $i$th feature of $j$th sample at $k$th omics data.

In order to bundle them into a tensor, we applied SVD to them as
$$
x_{ijk} = \sum_{\ell} \lambda^{(k)}_\ell u_{\ell i_k} v^{(k)}_{\ell j}
$$

Then apply HOSVD to $v^{(k)}_{\ell j}$ as
$$
v^{(k)}_{\ell j} = \sum_{\ell_1} \sum_{\ell_2} \sum_{\ell_3} G(\ell_1 \ell_2 \ell_3)
u_{\ell_1 \ell } u_{\ell_2 j} u_{\ell_3 k}
$$
After identifying the $u_{\ell_2j}$ and $u_{\ell_3 k}$ of interest, we can compute $u_{\ell i_k}$ as
$$
u_{\ell_2 i_k} = \sum_j u_{\ell_2 j} x_{i_k j k}
$$
Then $i_k$ can be selected as usual. 

## When features are shared.

![Projection when features are shared](./fig28.jpg)

Suppose we have multiple sets of samples as
$$
x_{ij_k k} \in \mathbb{R}^{N \times M_k \times K}
$$

In order to bundle them into a tensor, we apply SVD to $x_{i j_k k}$ as
$$
x_{i j_k k} = \sum_\ell \lambda^{(k)}_\ell u^{(k)}_{\ell i} u_{\ell j_k}
$$
HOSVD is applied to $u^{(k)}_{\ell i}$ as
$$
u^{(k)}_{\ell i} = \sum_{\ell_1} \sum_{\ell_2} \sum_{\ell_3}
G(\ell_1 \ell_2 \ell_3) u_{\ell_1 \ell} u_{\ell_2 i} u_{\ell_3 k}
$$
$u_{\ell_2 j_k}$ is generated as
$$
u_{\ell_2 j_k} = \sum_i u_{\ell_2 i}x_{i j_k k}
$$

After identifying $u_{\ell_2 j_k}$s of interest, we select $i$s using $u_{\ell_2 i}$.


```{r}
sessionInfo()
```

---
title: "Enrichment"
author:
- name: Y-h. Taguchi
  affiliation:  Department of Physics, Chuo University, Tokyo 112-8551, Japan
  email: tag@granular.com
output:   
    BiocStyle::html_document:
    toc: true
bibliography: references.bib
vignette: >
  %\VignetteIndexEntry{Enrichment}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  crop = NULL,
  comment = "#>"
)
```



```{r setup}
library(TDbasedUFE)
library(TDbasedUFEadv)
library(RTCGA.rnaseq)
library(RTCGA.clinical)
```

# Introduction

It might be helpful to demonstrate how to evaluate selected genes by enrichment analysis. Here, we show some of useful tools applied to the output from TDbasedUFEadv
In order foe this, we reproduce one example in "How to use TDbasedUFEadv" as follows.

``` {r}
Multi <- list(
  BLCA.rnaseq[seq_len(100), 1 + seq_len(1000)],
  BRCA.rnaseq[seq_len(100), 1 + seq_len(1000)],
  CESC.rnaseq[seq_len(100), 1 + seq_len(1000)],
  COAD.rnaseq[seq_len(100), 1 + seq_len(1000)]
)
Z <- prepareTensorfromList(Multi, 10L)
Z <- aperm(Z, c(2, 1, 3))
Clinical <- list(BLCA.clinical, BRCA.clinical, CESC.clinical, COAD.clinical)
Multi_sample <- list(
  BLCA.rnaseq[seq_len(100), 1, drop = FALSE],
  BRCA.rnaseq[seq_len(100), 1, drop = FALSE],
  CESC.rnaseq[seq_len(100), 1, drop = FALSE],
  COAD.rnaseq[seq_len(100), 1, drop = FALSE]
)
# patient.stage_event.tnm_categories.pathologic_categories.pathologic_m
ID_column_of_Multi_sample <- c(770, 1482, 773, 791)
# patient.bcr_patient_barcode
ID_column_of_Clinical <- c(20, 20, 12, 14)
Z <- PrepareSummarizedExperimentTensor(
  feature = colnames(ACC.rnaseq)[1 + seq_len(1000)],
  sample = array("", 1), value = Z,
  sampleData = prepareCondTCGA(
    Multi_sample, Clinical,
    ID_column_of_Multi_sample, ID_column_of_Clinical
  )
)
HOSVD <- computeHosvd(Z)
cond <- attr(Z, "sampleData")
index <- selectFeatureProj(HOSVD, Multi, cond, de = 1e-3, input_all = 3) # Batch mode
head(tableFeatures(Z, index))
genes <- unlist(lapply(strsplit(tableFeatures(Z, index)[, 1], "|",
  fixed = TRUE
), "[", 1))
entrez <- unlist(lapply(strsplit(tableFeatures(Z, index)[, 1], "|",
  fixed = TRUE
), "[", 2))
```

# Enrichr 

Enrichr[@Enrichr] is one of tools that often provides us significant results 
toward genes selected by TDbasedUFE and TDbasedUFEadv.

```{r enrichr-example, eval=FALSE}
if (!requireNamespace("enrichR", quietly = TRUE)) {
    stop("The enrichR package is required to run this example.")
}

enrichR::setEnrichrSite("Enrichr")

dbs <- c(
    "GO_Molecular_Function_2015",
    "GO_Cellular_Component_2015",
    "GO_Biological_Process_2015"
)

enriched <- enrichR::enrichr(genes, dbs)

enrichR::plotEnrich(
    enriched$GO_Biological_Process_2015,
    showTerms = 20,
    numChar = 40,
    y = "Count",
    orderBy = "P.value"
)
```

Enrichr can provide you huge number of enrichment analyses, 
many of which have good compatibility with the genes selected by
TDbasedUFE as well as TDbasedUFEadv by the experience. 
Please check Enrichr's web site to see what kinds of enrichment
analyses can be done.

# STRING

STRING[@STRING] is enrichment analyses based upon protein-protein interaction, 
which is known to provide often significant results toward genes selected by
TDbasedUFE as well as TDbasedUFEadv.

```{r stringdb-example, eval=FALSE}
if (!requireNamespace("STRINGdb", quietly = TRUE)) {
    stop("The STRINGdb package is required to run this example.")
}

string_cache <- tools::R_user_dir("TDbasedUFEadv", which = "cache")
dir.create(string_cache, recursive = TRUE, showWarnings = FALSE)

string_db <- STRINGdb::STRINGdb$new(
    version = "11.5",
    species = 9606,
    score_threshold = 200,
    network_type = "full",
    input_directory = string_cache
)

example1_mapped <- string_db$map(
    data.frame(genes = genes),
    "genes",
    removeUnmappedRows = TRUE
)

hits <- example1_mapped$STRING_id
string_db$plot_network(hits)
```

# enrichplot

Although these above can provide us enough number of information to evaluate 
the genes selected by TDbasedUFE as well as TDbasedUFEadv, one might need 
all one package for which one does not how to decide which category must
be evaluated in enrichment analysis.

In this case, we would recommend Metascape[@Metascape] that unfortunately  
does not have the ways approached from R. Thus, we recommend RITAN as
an alternative. It can list significant ones among multiple categories.


```{r dose-example}
if (
    requireNamespace("DOSE", quietly = TRUE) &&
    requireNamespace("enrichplot", quietly = TRUE) &&
    requireNamespace("gson", quietly = TRUE) &&
    requireNamespace("ggplot2", quietly = TRUE)
) {
    edo <- DOSE::enrichDGN(entrez)

    enrichplot::dotplot(edo, showCategory = 30) +
        ggplot2::ggtitle("dotplot for ORA")
} else {
    message(
        "Skipping DOSE/enrichplot example because one or more optional packages ",
        "are not installed: DOSE, enrichplot, gson, ggplot2."
    )
}
```
```{r}
sessionInfo()
```

---
title: "How to use TDbasedUFEadv"
author:
- name: Y-h. Taguchi
  affiliation:  Department of Physics, Chuo University, Tokyo 112-8551, Japan
  email: tag@granular.com
output:   
    BiocStyle::html_document:
    toc: true
bibliography: references.bib
vignette: >
  %\VignetteIndexEntry{How to use TDbasedUFEadv}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  crop = NULL,
  comment = "#>"
)
```

```{r setup}
library(TDbasedUFEadv)
library(Biobase)
library(RTCGA.rnaseq)
library(TDbasedUFE)
library(MOFAdata)
library(TDbasedUFE)
library(RTCGA.clinical)
```
# Installation

``` {r, eval = FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("TDbasedUFEadv")
```



# Integrated analyses of two omics profiles 


Here is a flowchart how we can make use of individual functions in 
[TDbasedUFE](https://doi.org/doi:10.18129/B9.bioc.TDbasedUFE) and TDbasedUFEadv.

![Relationship among functions in TDbasedUFE and TDbasedUFEadv](./flowchart.jpg)



## When features are shared.

In order to make use of [TDbasedUFE](https://doi.org/doi:10.18129/B9.bioc.TDbasedUFE) for the drug repositioning, we previously
proposed[@Taguchi2017] the integrated analysis of two gene expression profiles, 
each of which is composed of gene expression of drug treated one and disease 
one. At first, we try to prepare two omics profiles, expDrug and expDisease, 
that represent gene expression profiles of cell lines treated by various drugs
and a cell line of diseases by
``` {r}
Cancer_cell_lines <- list(ACC.rnaseq, BLCA.rnaseq, BRCA.rnaseq, CESC.rnaseq)
Drug_and_Disease <- prepareexpDrugandDisease(Cancer_cell_lines)
expDrug <- Drug_and_Disease$expDrug
expDisease <- Drug_and_Disease$expDisease
rm(Cancer_cell_lines)
```
expDrug is taken from RTCGA package and those associated with Drugs based upon 
[@Ding2016].  Those files are listed in drug_response.txt included in Clinical
drug responses at http://lifeome.net/supp/drug_response/.
expDisease is composed of files in BRCA.rnaseq, but not included in expDrug
(For more details, see source code of prepareexpDrugandDisease).
Then prepare a tensor as
```{r}
Z <- prepareTensorfromMatrix(
  exprs(expDrug[seq_len(200), seq_len(100)]),
  exprs(expDisease[seq_len(200), seq_len(100)])
)
sample <- outer(
  colnames(expDrug)[seq_len(100)],
  colnames(expDisease)[seq_len(100)], function(x, y) {
    paste(x, y)
  }
)
Z <- PrepareSummarizedExperimentTensor(
  sample = sample, feature = rownames(expDrug)[seq_len(200)], value = Z
)
```
In the above, sample are pairs of file IDs taken from expDrug and expDisease. 
Since full data cannot be treated because of memory restriction, we restricted 
the first two hundred features and the first one hundred samples, respectively 
(In the below, we will introduce how to deal with the full data sets).

Then HOSVD is applied to a tensor as
``` {r}
HOSVD <- computeHosvd(Z)
```
Here we tries to find if Cisplatin causes distinct expression  (0: cell lines
treated with drugs other than Cisplatin, 1: cell lines treated with Cisplatin)
and those between two classes (1 vs 2) of BRCA (in this case, there are no
meaning of two classes) within top one hundred samples.
``` {r}
Cond <- prepareCondDrugandDisease(expDrug)
cond <- list(NULL, Cond[, colnames = "Cisplatin"][seq_len(100)], rep(1:2, each = 50))
```
Then try to select singular value vectors attributed to objects.
When you try this vignettes, although you can do it in the interactive 
mode (see below), here we assume that you have already finished the selection. 

```{r}
input_all <- selectSingularValueVectorLarge(HOSVD,cond,input_all=c(2,9)) #Batch mode
```

In the case you prefer to select by yourself  you can execute interactive mode.
```
input_all <- selectSingularValueVectorLarge(HOSVD,cond)
```
When you can see ``Next'', ``Prev'', and ``Select'' radio buttons by which you 
can performs selection as well as histogram and standard deviation optimization
by which you can verify the success of selection interactively.


Next we select which genes' expression is altered by Cisplatin.
```{r }
index <- selectFeature(HOSVD,input_all,de=0.05)
```

You might need to specify suitable value for de which is initial value of
standard deviation. 

Then we get the following plot.



Finally, list the genes selected as those associated with distinct expression.
```{r}
head(tableFeatures(Z,index))
```
```{r}
rm(Z)
rm(HOSVD)
detach("package:RTCGA.rnaseq")
rm(SVD)
```
The described methods were frequently used
in the studies[@Taguchi2017a] [@Taguchi2018]  [@Taguchi2020] by maintainers.

### Reduction of required memory using partial summation.

In the case that  there are large number of features, it is impossible to apply
HOSVD to a full tensor (Then we have reduced the size of tensor).
In this case, we apply SVD instead of HOSVD to matrix 
generated from a tensor as follows.
In contrast to the above where only top two hundred features and top one hundred 
samples are included, the following one includes all features and all samples since
it can save required memory because partial summation of features.
``` {r}
SVD <- computeSVD(exprs(expDrug), exprs(expDisease))
Z <- t(exprs(expDrug)) %*% exprs(expDisease)
sample <- outer(
  colnames(expDrug), colnames(expDisease),
  function(x, y) {
    paste(x, y)
  }
)
Z <- PrepareSummarizedExperimentTensor(
  sample = sample,
  feature = rownames(expDrug), value = Z
)
```

Nest select singular value vectors attributed to drugs and cell lines then 
identify features associated with altered expression by treatment of
Cisplatin as well as distinction between two classes. Again, it included 
all samples for expDrug and expDisease.
``` {r}
cond <- list(NULL,Cond[,colnames="Cisplatin"],rep(1:2,each=dim(SVD$SVD$v)[1]/2))
```

Next we select singular value vectors and optimize standard deviation 
as batch mode 
```{r}
index_all <- selectFeatureRect(SVD,cond,de=c(0.01,0.01),
                               input_all=3) #batch mode
```
Again you need to select suitable de by trials and errors.

For interactive mode, one should do
```
index_all <- selectFeatureRect(SVD,cond,de=c(0.01,0.01))
```
but it is not possible in vignettes that does not allow interactive mode.


Then you can see selected features as 
```{r}
head(tableFeatures(Z,index_all[[1]]))
head(tableFeatures(Z,index_all[[2]]))
```
The upper one is  for distinct expression between cell lines treated with 
Cisplatin and other cell lines and the lower one is for distinct expression 
between two classes of BRCA cell lines.

Although they are highly coincident, not fully same ones (Row: expDrug, 
column:expDisease).
```{r}
table(index_all[[1]]$index,index_all[[2]]$index)
```

Confusion matrix of features selected between expDrug and expDisease.

The described methods were frequently used in the studies[@Taguchi2019a] by maintainers.

```{r}
rm(Z)
rm(SVD)
```

## When samples are shared 

The above procedure can be used when two omics data that shares samples must be integrated.
Prepare data set as
```{r}
data("CLL_data")
data("CLL_covariates")
```

(see vignettes QuickStart in [TDbasedUFE](https://doi.org/doi:10.18129/B9.bioc.TDbasedUFE) for more details about this 
data set).
Generate tensor from matrix as in the above, but since not features but 
samples are shared between two matrices,
the resulting Z has samples as features and features as samples, respectively.
```{r}
Z <- prepareTensorfromMatrix(
  t(CLL_data$Drugs[seq_len(200), seq_len(50)]),
  t(CLL_data$Methylation[seq_len(200), seq_len(50)])
)
Z <- prepareTensorRect(
  sample = colnames(CLL_data$Drugs)[seq_len(50)],
  feature = list(
    Drugs = rownames(CLL_data$Drugs)[seq_len(200)],
    Methylatiion = rownames(CLL_data$Methylation)[seq_len(200)]
  ),
  sampleData = list(CLL_covariates$Gender[seq_len(50)]),
  value = Z
)
```

HOSVD was applied to Z as
```{r}
HOSVD <- computeHosvd(Z)
```

```{r}
cond <- list(attr(Z,"sampleData")[[1]],NULL,NULL)
```
Condition is distinction between male and female 
(see QucikStart in TDbasedUFE package).
Then try to find singular value vectors distinct between  male and female 
in interactive mode.
```{r}
index_all <- selectFeatureTransRect(HOSVD,cond,de=c(0.01,0.01),
                                    input_all=8) #batch mode
```
In the above, selection was supposed to be performed before executing the
above, since  vignettes does not allow interactive mode.
 In actual, you need to execute it in interactive mode
```
index_all <- selectFeatureTransRect(HOSVD,cond,de=c(0.01,0.01))
```
and try to select iteratively. Selected features can be shown in the below. 

```{r}
head(tableFeaturesSquare(Z,index_all,1))
head(tableFeaturesSquare(Z,index_all,2))
```

This method was used in the studies[@Taguchi2019] by the maintainer. 

###  Reduction of required memory using partial summation.

As in the case where two omics profiles share features, in the case where two
omics data share the samples, we can also take an alternative approach where 
SVD is applied to an matrix generated from a tensor by taking partial summation.
```{r}
SVD <- computeSVD(t(CLL_data$Drugs), t(CLL_data$Methylation))
Z <- CLL_data$Drugs %*% t(CLL_data$Methylation)
sample <- colnames(CLL_data$Methylation)
Z <- prepareTensorRect(
  sample = sample,
  feature = list(rownames(CLL_data$Drugs), rownames(CLL_data$Methylation)),
  value = array(NA, dim(Z)), sampleData = list(CLL_covariates[, 1])
)
```
Condition is also distinction between male  (m) and female (f).
```{r}
cond <- list(NULL,attr(Z,"sampleData")[[1]],attr(Z,"sampleData")[[1]])
```
In order to apply the previous function to SVD, we exchange feature singular 
value vectors with sample singular value vectors. 
```{r}
SVD <- transSVD(SVD)
```
Then try to find which sample singular value vectors should be selected and
which features are selected based upon feature singular value vectors
corresponding to selected sample feature vectors.
Although I do not intend to repeat whole process, we decided to select the 
sixth singular value vectors which are some what distinct between male 
and female. Since package does not allow us interactive mode, we place here batch mode.

```{r}
index_all <- selectFeatureRect(SVD,cond,de=c(0.5,0.5),input_all=6) #batch mode
```

In the real usage, we can activate 
selectFeatureRect in interactive mode as well.
```
index_all <- selectFeatureRect(SVD,cond,de=c(0.5,0.5))
```














Then we can list the Drugs and Methylation sites selected as being distinct 
between male and female.

```{r}
head(tableFeaturesSquare(Z,index_all,1))
head(tableFeaturesSquare(Z,index_all,2))
```



This method was used in many studies[@Taguchi2018a] [@Taguchi2020a] by maintainer. 


# Integrated analysis of multiple omics data

Here is a flowchart how we can make use of individual functions in TDbasedUFE and TDbasedUFEadv.

![Relationship among functions in TDbasedUFE and TDbasedUFEadv](./flowchart2.jpg)



## When samples are shared

As an alternative approach that can integrate multiple omics that share sample, 
we propose the method that makes use of projection provided by SVD.

We prepare a tensor that is a bundle of the first ten singular value vectors 
generated by applying SVD to individual omics profiles.

```{r}
data("CLL_data")
data("CLL_covariates")
Z <- prepareTensorfromList(CLL_data, 10L)
Z <- PrepareSummarizedExperimentTensor(
  feature = character("1"),
  sample = array(colnames(CLL_data$Drugs), 1), value = Z,
  sampleData = list(CLL_covariates[, 1])
)
```
Then HOSVD was applied to a tensor
```{r}
HOSVD <- computeHosvd(Z,scale=FALSE)
```
Next we select singular value vectors attributed to samples.
In order to select those distinct between male (m) and female (f),
we set conditions as
```{r}
cond <- list(NULL,attr(Z,"sampleData")[[1]],seq_len(4))
```
But here in order to include TDbasedUFEadv into package, we are forced to
execute function as batch mode as
```{r}
input_all <- selectSingularValueVectorLarge(HOSVD,
  cond,
  input_all = c(12, 1)
) # batch mode
```
Interactive more can be activated as
```
input_all <- selectSingularValueVectorLarge(HOSVD,cond)
```
Although we do not intend to repeat how to use menu in interactive mode, please 
select the 12th one and the third one. 

Finally,  we perform the following function to select features in individual 
omics profiles in an batch mode, 
since packaging does not allow interactive mode.

``` {r}
HOSVD$U[[1]] <- HOSVD$U[[2]]
index_all <- selectFeatureSquare(HOSVD, input_all, CLL_data,
  de = c(0.5, 0.1, 0.1, 1), interact = FALSE
) # Batch mode
```

In actual usage, you can activate interactive mode as
``` 
HOSVD$U[[1]] <- HOSVD$U[[2]]
index_all <- selectFeatureSquare(HOSVD, input_all, CLL_data,
  de = c(0.5, 0.1, 0.1, 1)
)
```

Finally, we list the selected features for four omics profiles that share samples.

```{r}
for (id in c(1:4))
{
  attr(Z, "feature") <- rownames(CLL_data[[id]])
  print(tableFeatures(Z, index_all[[id]]))
}
```

This method was used in many studies[@Taguchi2022] by maintainer. 


## When features are shared

Now we discuss what to do when multiple omics data share not samples but 
features. We prepare data set from RTCGA.rnaseq as follows, with retrieving 
reduced partial sets from four ones. One should notice that RTCGA is an old
package from TCGA (as for 2015). I used it only for demonstration purpose.
If you would like to use TCGA for your research, I recommend you to use
more modern packages, e.g., curatedTCGAData in Bioconductor.
```{r}
library(RTCGA.rnaseq) #it must be here, not in the first chunk
Multi <- list(
  BLCA.rnaseq[seq_len(100), 1 + seq_len(1000)],
  BRCA.rnaseq[seq_len(100), 1 + seq_len(1000)],
  CESC.rnaseq[seq_len(100), 1 + seq_len(1000)],
  COAD.rnaseq[seq_len(100), 1 + seq_len(1000)]
)
```
Multi includes four objects, each of which is matrix that represent 100 samples (rows) and 1000 (features). Please note it is different from usual cases where columns and rows are features and samples, respectively. They are marge into tensor as follows
```{r}
Z <- prepareTensorfromList(Multi,10L)
Z <- aperm(Z,c(2,1,3))
```
The function, prepareTeansorfromList which was used in the previous subsection
where samples are shared, can be used as it is. However, the first and second 
modes of a tensor must be exchanged by aperm function for the latter analyses,
because of the difference as mentioned in the above. Then tensor object 
associated with various information is generated as usual as follows and 
HOSVD was applied to it. 
``` {r}
Clinical <- list(BLCA.clinical, BRCA.clinical, CESC.clinical, COAD.clinical)
Multi_sample <- list(
  BLCA.rnaseq[seq_len(100), 1, drop = FALSE],
  BRCA.rnaseq[seq_len(100), 1, drop = FALSE],
  CESC.rnaseq[seq_len(100), 1, drop = FALSE],
  COAD.rnaseq[seq_len(100), 1, drop = FALSE]
)
# patient.stage_event.tnm_categories.pathologic_categories.pathologic_m
ID_column_of_Multi_sample <- c(770, 1482, 773, 791)
# patient.bcr_patient_barcode
ID_column_of_Clinical <- c(20, 20, 12, 14)
Z <- PrepareSummarizedExperimentTensor(
  feature = colnames(ACC.rnaseq)[1 + seq_len(1000)],
  sample = array("", 1), value = Z,
  sampleData = prepareCondTCGA(
    Multi_sample, Clinical,
    ID_column_of_Multi_sample, ID_column_of_Clinical
  )
)
HOSVD <- computeHosvd(Z)
```
In order to see which singular value vectors attributed to samples are used for the selection of singular value vectors attributed to features, we need to assign sample conditions.
```{r}
cond<- attr(Z,"sampleData")
```

Since package does not allow us to include interactive mode, we place here batch mode as follows.
Finally, selected feature are listed as follows.
``` {r}
index <- selectFeatureProj(HOSVD,Multi,cond,de=1e-3,input_all=3) #Batch mode
head(tableFeatures(Z,index))
```
In actual, you can activate interactive mode as 

```
par(mai=c(0.3,0.2,0.2,0.2))
index <- selectFeatureProj(HOSVD,Multi,cond,de=1e-3)
```
Although we do not intend to explain how to use menu interactively, 
we select the third singular value vectors as shown in above.

This method was used in many studies[@Taguchi2021] by maintainer. 


```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selectFeatureTransRect.R
\name{selectFeatureTransRect}
\alias{selectFeatureTransRect}
\title{Select features for a tensor generated from two matrices that
 share samples.}
\usage{
selectFeatureTransRect(
  HOSVD,
  cond,
  de = rep(1e-04, 2),
  p0 = 0.01,
  breaks = 100L,
  input_all = NULL
)
}
\arguments{
\item{HOSVD}{HOSVD}

\item{cond}{list of conditions}

\item{de}{initial values for optimization of standard deviation}

\item{p0}{threshold value for the significance}

\item{breaks}{number of bins of the histogram of P-values}

\item{input_all}{The selected singular value vectors attributed to samples.
if NULL, interactive mode}
}
\value{
list of logical vector that represent if the individual features
are selected and P-values.
}
\description{
Select features for a tensor generated from two matrices that
 share samples.
}
\examples{
library(TDbasedUFE)
set.seed(0)
matrix1 <- matrix(runif(1000),20) #row features, column samples
matrix2 <- matrix(runif(2000),40) #row features, column samples
Z <- prepareTensorfromMatrix(t(matrix1),t(matrix2))
Z <- prepareTensorRect(sample=as.character(seq_len(50)),
feature=list(as.character(seq_len(20)),as.character(seq_len(40))),
sampleData=list(rep(seq_len(2),each=25)),value=Z)
HOSVD <- computeHosvd(Z)
cond <- list(attr(Z,"sampleData")[[1]],NULL,NULL)
index_all <- selectFeatureTransRect(HOSVD,cond,de=c(0.1,0.1),
input_all=2,p0=1e-10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selectFeatureRect.R
\name{selectFeatureRect}
\alias{selectFeatureRect}
\title{Select features through the selection  of singular value vectors}
\usage{
selectFeatureRect(
  SVD,
  cond,
  de = rep(1e-04, 2),
  p0 = 0.01,
  breaks = 100L,
  input_all = NULL
)
}
\arguments{
\item{SVD}{SVD computed from matrix generated by partial summation of a tensor}

\item{cond}{Condition to select singular value vectors}

\item{de}{Initial values to be used for optimization of standard deviation}

\item{p0}{Threshold value for the significance}

\item{breaks}{Number of bins of histogram of P-values}

\item{input_all}{The ID of selected singular value vectors. If it is null, 
interactive mode is activated.}
}
\value{
List of lists that includes P-vales as well as if individual
 features selected.
}
\description{
Select features through the selection  of singular value vectors
}
\examples{
set.seed(0)
matrix1 <- matrix(runif(2000),200)
matrix2 <- matrix(runif(4000),200)
SVD <- computeSVD(matrix1,matrix2)
index_all <- selectFeatureRect(SVD,
list(NULL,rep(seq_len(2),each=5),rep(seq_len(2),each=10)),de=rep(0.5,2),
input_all=1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepareTensorRect.R
\name{prepareTensorRect}
\alias{prepareTensorRect}
\title{Prepare tensor generated from two matrices that share samples}
\usage{
prepareTensorRect(
  sample,
  feature,
  value,
  featureRange = GRanges(NULL),
  sampleData = list(NULL)
)
}
\arguments{
\item{sample}{Character vector of sample names}

\item{feature}{list of features from two matrices}

\item{value}{array, contents of}

\item{featureRange}{Genomic Ranges to be associated with features}

\item{sampleData}{List of conditional labeling associated with samples}
}
\value{
Tensor generated from two matrices that share samples
}
\description{
Prepare tensor generated from two matrices that share samples
}
\examples{
matrix1 <- matrix(runif(1000),200) #row features, column samples
matrix2 <- matrix(runif(2000),400) #row features, column samples
Z <- prepareTensorfromMatrix(t(matrix1),t(matrix2))
Z <- prepareTensorRect(sample=as.character(seq_len(50)),
feature=list(as.character(seq_len(200)),as.character(seq_len(400))),
sampleData=list(rep(seq_len(2),each=25)),value=Z)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepareCondDrugandDisease.R
\name{prepareCondDrugandDisease}
\alias{prepareCondDrugandDisease}
\title{Prepare condition matrix for expDrug}
\usage{
prepareCondDrugandDisease(expDrug)
}
\arguments{
\item{expDrug}{input gene expression profile}
}
\value{
Condition matrix for expDrug
}
\description{
Prepare condition matrix for expDrug
}
\examples{
\donttest{
library(RTCGA.rnaseq)
Cancer_cell_lines <- list(ACC.rnaseq,BLCA.rnaseq,BRCA.rnaseq)
Drug_and_Disease <- prepareexpDrugandDisease(Cancer_cell_lines)
Cond <- prepareCondDrugandDisease(Drug_and_Disease$expDrug)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepareCondTCGA.R
\name{prepareCondTCGA}
\alias{prepareCondTCGA}
\title{Prepare Sample label for TCGA data}
\usage{
prepareCondTCGA(
  Multi_sample,
  Clinical,
  ID_column_of_Multi_sample,
  ID_column_of_Clinical
)
}
\arguments{
\item{Multi_sample}{list of sample ids}

\item{Clinical}{List of clinical data matrix from RTCGA.clinical}

\item{ID_column_of_Multi_sample}{Column numbers used for conditions}

\item{ID_column_of_Clinical}{Column numbers that include corresponding
sample ids in clinical data}
}
\value{
list of sample labels
}
\description{
Prepare Sample label for TCGA data
}
\examples{
library(RTCGA.clinical)
library(RTCGA.rnaseq)
Clinical <- list(BLCA.clinical, BRCA.clinical, CESC.clinical, COAD.clinical)
Multi_sample <- list(
  BLCA.rnaseq[seq_len(100), 1, drop = FALSE],
  BRCA.rnaseq[seq_len(100), 1, drop = FALSE],
  CESC.rnaseq[seq_len(100), 1, drop = FALSE],
  COAD.rnaseq[seq_len(100), 1, drop = FALSE]
)
ID_column_of_Multi_sample <- c(770, 1482, 773, 791)
ID_column_of_Clinical <- c(20, 20, 12, 14)
cond <- prepareCondTCGA(
  Multi_sample, Clinical,
  ID_column_of_Multi_sample, ID_column_of_Clinical
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeSVD.R
\name{computeSVD}
\alias{computeSVD}
\title{Title Perform SVD toward reduced matrix generated from a tensor with 
partial summation}
\usage{
computeSVD(matrix1, matrix2, dim = 10L, scale = TRUE)
}
\arguments{
\item{matrix1}{The first original matrix that generates a tensor}

\item{matrix2}{The second original matrix that generates a tensor}

\item{dim}{The number of singular value vectors to be computed}

\item{scale}{If matrix should be scaled or not}
}
\value{
Singular value vectors attributed to two sets of objects associated 
with singular value vectors attributed to features, by multiplying
}
\description{
Title Perform SVD toward reduced matrix generated from a tensor with 
partial summation
}
\examples{
matrix1 <- matrix(runif(200),20)
matrix2 <- matrix(runif(400),20)
SVD <- computeSVD(matrix1,matrix2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepareTensorfromMatrix.R
\name{prepareTensorfromMatrix}
\alias{prepareTensorfromMatrix}
\title{Generate tensor from two matrices}
\usage{
prepareTensorfromMatrix(matrix1, matrix2)
}
\arguments{
\item{matrix1}{the first input matrix}

\item{matrix2}{the second input matrix}
}
\value{
A tensor generated from the first and second matrices
}
\description{
Generate tensor from two matrices
}
\examples{
Z <- prepareTensorfromMatrix(matrix(runif(100),10),matrix(runif(100),10))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepareexpDrugandDisease.R
\name{prepareexpDrugandDisease}
\alias{prepareexpDrugandDisease}
\title{Generating gene expression of drug treated cell lines and a disease
cell line}
\usage{
prepareexpDrugandDisease(Cancer_cell_lines)
}
\arguments{
\item{Cancer_cell_lines}{<- list(ACC.rnaseq,BLCA.rnaseq,BRCA.rnaseq)
list that includes individual data set from RTCGA.rnaseq}
}
\value{
list of expDrug and expDisease
}
\description{
Generating gene expression of drug treated cell lines and a disease
cell line
}
\examples{
 \donttest{
library(RTCGA.rnaseq)
Cancer_cell_lines <- list(ACC.rnaseq,BLCA.rnaseq,BRCA.rnaseq)
Drug_and_Disease <- prepareexpDrugandDisease(Cancer_cell_lines)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selectFeatureProj.R
\name{selectFeatureProj}
\alias{selectFeatureProj}
\title{Select feature when projection strategy is employed for the
 case where features are shared with multiple omics profiles}
\usage{
selectFeatureProj(
  HOSVD,
  Multi,
  cond,
  de = 1e-04,
  p0 = 0.01,
  breaks = 100L,
  input_all = NULL
)
}
\arguments{
\item{HOSVD}{HOSVD}

\item{Multi}{list of omics profiles, row: sample, column: feature}

\item{cond}{list of conditions for individual omics profiles}

\item{de}{initial value for optimization of standard deviation}

\item{p0}{Threshold P-value}

\item{breaks}{The number of bins of histogram of P-values}

\item{input_all}{The number of selected feature. if null, interactive mode
is activated}
}
\value{
list composed of logical vector that represent which features are selected and p-values
}
\description{
Select feature when projection strategy is employed for the
 case where features are shared with multiple omics profiles
}
\examples{
library(TDbasedUFE)
Multi <- list(matrix(runif(1000),10),matrix(runif(1000),10),
matrix(runif(1000),10),matrix(runif(1000),10))
Z <- prepareTensorfromList(Multi,10L)
Z <- aperm(Z,c(2,1,3))
Z <- PrepareSummarizedExperimentTensor(feature =as.character(1:10),
                                      sample=array("",1),value=Z)
HOSVD <- computeHosvd(Z)
cond <- rep(list(rep(1:2,each=5)),4)
index <- selectFeatureProj(HOSVD,Multi,cond,de=0.1,input_all=2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{TensorRect-class}
\alias{TensorRect-class}
\title{Class definitions}
\description{
Class definitions
}
\section{Slots}{

\describe{
\item{\code{sample}}{character.}

\item{\code{feature}}{list.}

\item{\code{value}}{array.}

\item{\code{featureRange}}{GRanges.}

\item{\code{sampleData}}{list.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TDbasedUFEadv-package.R
\docType{package}
\name{TDbasedUFEadv-package}
\alias{TDbasedUFEadv}
\alias{TDbasedUFEadv-package}
\title{TDbasedUFEadv: Advanced package of tensor decomposition based unsupervised feature extraction}
\description{
This is an advanced version of TDbasedUFE, which is a comprehensive package to perform Tensor decomposition based unsupervised feature extraction. In contrast to TDbasedUFE which can perform simple the feature selection and the multiomics analyses, this package can perform more complicated and advanced features, but they are not so popularly required. Only users who require more specific features can make use of its functionality.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/tagtag/TDbasedUFEadv}
  \item Report bugs at \url{https://github.com/tagtag/TDbasedUFEadv/issues}
}

}
\author{
\strong{Maintainer}: Y-h. Taguchi \email{tag@granular.com} (\href{https://orcid.org/0000-0003-0867-8986}{ORCID})

Authors:
\itemize{
  \item Y-h. Taguchi \email{tag@granular.com} (\href{https://orcid.org/0000-0003-0867-8986}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepareTensorfromList.R
\name{prepareTensorfromList}
\alias{prepareTensorfromList}
\title{Prepare tensor from a list that includes multiple profiles}
\usage{
prepareTensorfromList(Multi, proj_dim)
}
\arguments{
\item{Multi}{a list that includes multiple profiles}

\item{proj_dim}{the number of projection dimensions}
}
\value{
a tensor as a bundle of singular value vectors obtained by 
applying  SVD to individual omics
}
\description{
Prepare tensor from a list that includes multiple profiles
}
\examples{
library(MOFAdata)
data("CLL_data")
data("CLL_covariates")
Z <- prepareTensorfromList(CLL_data,10L)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transSVD.R
\name{transSVD}
\alias{transSVD}
\title{Convert SVD to that for the case where 
samples are shared between two matrices}
\usage{
transSVD(SVD)
}
\arguments{
\item{SVD}{input SVD object generated from computeSVD function}
}
\value{
converted SVD objects
}
\description{
Convert SVD to that for the case where 
samples are shared between two matrices
}
\examples{
matrix1 <- matrix(runif(200),20)
matrix2 <- matrix(runif(400),20)
SVD <- computeSVD(matrix1,matrix2)
SVD <- transSVD(SVD)
}
